package net.sergeych.kiloparsec.adapter

import kotlinx.coroutines.channels.ReceiveChannel

/**
 * Multiplatform implementation of an internet address.
 * Notice to implementors. It must provide correct and effective [equals] and [hashCode].
 */
interface NetworkAddress {
    val host: String
    val port: Int
}

/**
 * Multiplatform datagram abstraction
 */
interface Datagram {
    val message: UByteArray
    val address: NetworkAddress
    suspend fun respondWith(message: UByteArray)
}
interface DatagramReceiver {

    val incoming: ReceiveChannel<Datagram>
    suspend fun send(message: UByteArray, networkAddress: NetworkAddress)
    @Suppress("unused")
    suspend fun send(message: UByteArray, datagramAddress: String) {
        send(message, networkAddressOf(datagramAddress))
    }

    suspend fun send(message: UByteArray,host: String,port: Int) =
        send(message, NetworkAddress(host,port))
    fun close()
}

expect fun networkAddressOf(address: String): NetworkAddress
expect fun NetworkAddress(host: String,port: Int): NetworkAddress
